import ctypes
import re
import telnetlib
import time
from datetime import datetime

import serial
from PyQt5 import QtCore

import exceptions
from config import Config


class IntegratorGpio(QtCore.QThread):
    on_message = QtCore.pyqtSignal(dict)

    def __init__(self, callback):
        super().__init__()
        self.lib = ctypes.WinDLL(Config.DLL_CONF)
        self.callback = callback
        _IN_ = 1
        _OUT_ = 2
        _BITMASK_ = 0xFFFFFFFF
        _ERR_ = 0xFFFFFFFF
        _OK_ = 0xFFFFFFFE

        initApiProto = ctypes.WINFUNCTYPE(ctypes.c_void_p)
        initApi = initApiProto(("EApiLibInitialize", self.lib))

        if initApi() == _ERR_:
            raise exceptions.DefaultException("Falhar ao iniciar API GPIO")
        else:
            print("API GPIO inicializado com sucesso...")

        getLevelProto = ctypes.WINFUNCTYPE(ctypes.POINTER(ctypes.c_uint32),
                                           ctypes.c_uint32,
                                           ctypes.c_uint32,
                                           ctypes.POINTER(ctypes.c_uint32))

        paramFlags = (_IN_, "id", 0), (_IN_, "bitmask", 0), (_IN_, "res", 0)
        self.getLevel = getLevelProto(("EApiGPIOGetLevel", self.lib), paramFlags)

        self.p1 = ctypes.c_uint32(Config.GPIO_PIN)
        self.p2 = ctypes.c_uint32(_BITMASK_)
        self.res = ctypes.c_uint32()

    def run(self):
        signal_detected = False
        while True:
            time.sleep(0.5)
            self.getLevel(self.p1, self.p2, ctypes.byref(self.res))
            if self.res.value == 0:
                if not signal_detected:
                    self.callback(1)
                    signal_detected = True
            else:
                signal_detected = False


class IntegratorTcp(QtCore.QThread):
    on_message = QtCore.pyqtSignal()

    def __init__(self, callback):
        super().__init__()
        self.callback = callback
        self.reset_macro = False

    def handle(self, result):
        columns = result.split(',')
        if len(columns):
            count = re.findall("\d+\.\d+", columns[-1])
            if count:
                value = float(count[0])
                if value > 0:
                    self.reset_macro = self.callback(value)
                else:
                    self.reset_macro = False

    def run(self):
        while True:
            try:
                tn = telnetlib.Telnet(Config.IP_MACHINE, Config.PORT_MACHINE, timeout=3)

                # leitura da macro
                command = Config.COMMAND_READ_MACRO + '\r\n'
                tn.write(command.encode())
                print('<< ' + str(datetime.now().time()) + ' - ' + Config.COMMAND_READ_MACRO + '')
                result = tn.read_until(b'\n')
                print(result.decode())
                self.handle(result.decode())

                # escrita da macro
                if self.reset_macro:
                    command = Config.COMMAND_WRITE_MACRO + '\r\n'
                    tn.write(command.encode())
                    print('<< ' + str(datetime.now().time()) + ' - ' + Config.COMMAND_WRITE_MACRO + '')
                    result = tn.read_until(b'\n')
                    print(result.decode())

                tn.close()
                time.sleep(5)

            except Exception as ex:
                print(ex)
                time.sleep(5)


class IntegratorSerial(QtCore.QThread):
    on_message = QtCore.pyqtSignal()

    def __init__(self, callback):
        super().__init__()
        self.callback = callback
        self.serial = None
        self.reset_macro = False

    def handle(self, result):
        columns = result.split(',')
        if len(columns):
            count = re.findall("\d+\.\d+", columns[-1])
            if count:
                value = float(count[0])
                if value > 0:
                    self.reset_macro = self.callback(value)
                else:
                    self.reset_macro = False

    def run(self):

        while True:
            try:
                self.serial = serial.Serial(port=Config.PORT_COM)
                time.sleep(0.5)
                if self.serial is None or not self.serial.isOpen():
                    print(">> porta serial inválida")
                    time.sleep(5)
                    continue

                command = Config.COMMAND_READ_MACRO + '\r\n'
                print('<< ' + str(datetime.now().time()) + ' - ' + Config.COMMAND_READ_MACRO + '')
                self.serial.write(command.encode())
                time.sleep(0.1)
                result = self.serial.read_all().decode()
                print(result)
                self.handle(result)

                # escrita da macro
                if self.reset_macro:
                    command = Config.COMMAND_WRITE_MACRO + '\r\n'
                    self.serial.write(command.encode())
                    print('<< ' + str(datetime.now().time()) + ' - ' + Config.COMMAND_WRITE_MACRO + '')
                    result = self.serial.read_all().decode()
                    print(result.decode())

            except Exception as ex:
                print(ex)

            finally:
                self.serial.close()
                time.sleep(2)
